@@ -22,6 +22,7 @@ import json
 import hashlib
 import base64
 import requests
+from tabulate import tabulate
 from io import BytesIO, StringIO
 
 class Sdata_Name_Exeption(Exception): pass
@@ -71,17 +72,28 @@ class Data(object):
         # self._uuid = None
         # self._name = None
         self._prefix = None
+
         # ToDo: add getter and setter for metadata
-        self.metadata = kwargs.get("metadata") or Metadata()
+        # self.metadata = kwargs.get("metadata") or Metadata()
+
+        self.metadata = Metadata()
 
         # set default sdata attributes
         self.metadata.add(self.SDATA_VERSION, __version__, dtype="str")
-        self.metadata.add(self.SDATA_NAME, "", dtype="str")
-        self.metadata.add(self.SDATA_UUID, "", dtype="str")
+        self.metadata.add(self.SDATA_NAME, "N.N.", dtype="str")
+        self.metadata.add(self.SDATA_UUID, uuid.uuid4(), dtype="str")
         self.metadata.add(self.SDATA_CTIME, now_utc_str(), dtype="str")
         self.metadata.add(self.SDATA_MTIME, now_utc_str(), dtype="str")
         self.metadata.add(self.SDATA_PARENT, "", dtype="str")
 
+        metadata = kwargs.get("metadata")
+        if metadata is not None:
+            # logger.debug("Data got Metadata {}".format(metadata))
+            if metadata and isinstance(metadata, Metadata):
+                for attribute in metadata.attributes.values():
+                    # logger.debug("Data.Metadata.add {0.name}:{0.value}".format(attribute))
+                    self.metadata.add(attribute)
+
         # auto correct
         if kwargs.get("auto_correct") is None or kwargs.get("auto_correct") is True:
             self.auto_correct = True
@@ -90,15 +102,19 @@ class Data(object):
 
         logger.debug("sdata: set auto_correct={}".format(self.auto_correct))
 
-        try:
-            self.uuid = kwargs.get("uuid") or uuid.uuid4() # store given uuid str or generate a new uuid
-        except Sdata_Uuid_Exeption as exp:
-            if self.auto_correct is True:
-                logger.warning("got invald uuid -> generate a new uuid")
-                self.uuid = uuid.uuid4()
-            else:
-                raise
-        self.name = kwargs.get("name") or "N.N."
+        if kwargs.get("uuid") is not None:
+            try:
+                self.uuid = kwargs.get("uuid") # store given uuid str or generate a new uuid
+            except Sdata_Uuid_Exeption as exp:
+                if self.auto_correct is True:
+                    logger.warning("got invald uuid -> generate a new uuid")
+                    self.uuid = uuid.uuid4()
+                else:
+                    raise
+
+        if kwargs.get("name") is not None:
+            self.name = kwargs.get("name")
+
         self.prefix = kwargs.get("prefix") or ""
         self._gen_default_attributes(kwargs.get("default_attributes") or self.ATTR_NAMES)
         self._group = OrderedDict()
@@ -107,6 +123,9 @@ class Data(object):
         self._description = ""
         self.description = kwargs.get("description", "")
 
+
+
+
     def update_mtime(self):
         """update modification time
 
@@ -857,6 +876,109 @@ class Data(object):
         data.table = df
         return data
 
+    def to_html(self, filepath, xlsx=True, style=None):
+        """export Data to html
+
+        :param filepath:
+        :param xlsx:
+        :param style:
+        :return:
+        """
+
+        table_values = self.df.head()
+        table_headers = self.df.columns
+
+        table_description_values = self.df.describe()
+        table_description_headers = self.df.describe().columns
+
+        metadata_values = self.metadata.df.head().values
+        metadata_headers = self.metadata.df.columns
+
+        if xlsx is True:
+            xlsx_tag = self.get_download_link()
+        else:
+            xlsx_tag = ""
+
+        param = {"title":"{0} [{1}]".format(self.osname,
+                                            self.uuid),
+                 "description":self.description,
+                 "metadata": tabulate(metadata_values, metadata_headers, tablefmt="html"),
+                 "table": tabulate(table_values, table_headers, tablefmt="html"),
+                 "table_description": tabulate(table_description_values, table_description_headers, tablefmt="html"),
+                 "xlsx_tag":xlsx_tag,
+                 "sdata":"created with sdata v{}.".format(__version__),
+                 "now":"{}".format(now_utc_str()),
+                 }
+
+        tmpl = """<!DOCTYPE html>
+<html lang="de">
+  <head>
+    <meta charset="utf-8">
+    <meta name="viewport" content="width=device-width, initial-scale=1.0">
+    <title>{title}</title>
+    <style>
+    h1 {{ 
+       background-color: #00FFFF77;
+       color: black;
+    }}
+    h2 {{ 
+       background-color: #00FFFF44;
+       color: black;
+    }}
+    h3 {{ 
+       background-color: #00FFFF11;
+       color: black;
+    }}
+    p {{
+    	color: black;
+    }}
+    table, th, td, caption {{
+    border: 1px solid #a0a0a0;
+    }}
+    
+    table {{
+      border-collapse: collapse;
+      border-spacing: 1em;
+      border-width: thin 0 0 thin;
+      margin: 0 0 1em;
+      table-layout: auto;
+      max-width: 100%;
+      text-align: right;
+    }}
+    th, td {{
+      font-weight: normal;
+      text-align: left;
+      border-spacing: 1em;
+      padding: .1em .3em;
+    }}
+    th, caption {{
+      background-color: #f1f3f4;
+      font-weight: 700;
+    }}
+    </style>
+  </head>
+  <body>
+    <h1>{title}</h1>
+    <h2>Download</h2>
+    <p">{xlsx_tag}</p>
+    <h2>Description</h2>
+    <p>{description}</p>
+    <h2>Metadata</h2>
+    {metadata}
+    <h2>Table</h2>
+    {table}
+    <h3>Table Description</h3>
+    {table_description}
+    <p>{sdata}</p>
+    <p>{now}</p>
+  </body>
+</html>""".format(**param)
+        try:
+            with open(filepath, "w") as fh:
+                fh.write(tmpl)
+        except Exception as exp:
+            raise
+
     def copy(self):
         """create a copy of the Data object
 
